/*
 *  TOPPERS Software
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *
 *  Copyright (C) 2007-2008 by TAKAGI Nobuhisa
 *  Copyright (C) 2010 by Meika Sugimoto
 *
 *  L쌠҂́Cȉ(1)`(4)̏𖞂ꍇɌC{\tgEF
 *  Ai{\tgEFAς̂܂ށDȉjgpEE
 *  ρEĔzziȉCpƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *      ܂C{\tgEFÃ[U܂̓Gh[ÛȂ闝
 *      RɊÂCL쌠҂TOPPERSvWFNg
 *      Ɛӂ邱ƁD
 * 
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̎gpړI
 *  ɑ΂K܂߂āCȂۏ؂sȂD܂C{\tgEF
 *  A̗pɂ蒼ړI܂͊ԐړIɐȂ鑹QɊւĂC
 *  ̐ӔC𕉂ȂD
 * 
 */
/*!
 *  \file   toppers/diagnostics.hpp
 *  \brief  ffɊւ錾`
 */
#ifndef TOPPERS_DIAGNOSTICS_HPP_
#define TOPPERS_DIAGNOSTICS_HPP_

#include <stdexcept>
#include "toppers/debug.hpp"
#include "toppers/gettext.hpp"
#include <boost/format.hpp>

namespace toppers
{

  struct text_line;

  class diagnostics_error : public std::runtime_error
  {
  public:
    diagnostics_error( std::string const& msg ) : std::runtime_error( msg ) {}
  };

  class normal_exit {};

  int get_error_count();
  int increment_error_count();
  void set_program_name( char const* name );
  std::string const& get_program_name();
  int set_error_abort_threshold( int thresh );
  void warning( const char* msg );
  void warning( text_line const& line, const char* msg );
  void error( const char* msg );
  void error( text_line const& line, const char* msg );
  void fatal( const char* msg );
  void fatal( text_line const& line, const char* msg );

  template < typename T1 >
    inline void warning( const char* str, T1 const& arg1 )
  {
    warning( ( boost::format( str ) % arg1 ).str().c_str() );
  }

  template < typename T1, typename T2 >
    inline void warning( const char* str, T1 const& arg1, T2 const& arg2 )
  {
    warning( ( boost::format( str ) % arg1 % arg2 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3 >
    inline void warning( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3 )
  {
    warning( ( boost::format( str ) % arg1 % arg2 % arg3 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4 >
    inline void warning( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 )
  {
    warning( ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 ).str().c_str() );
  }

  template < typename T1 >
    inline void warning( text_line const& line, const char* str, T1 const& arg1 )
  {
    warning( line, ( boost::format( str ) % arg1 ).str().c_str() );
  }

  template < typename T1, typename T2 >
    inline void warning( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2 )
  {
    warning( line, ( boost::format( str ) % arg1 % arg2 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3 >
    inline void warning( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3 )
  {
    warning( line, ( boost::format( str ) % arg1 % arg2 % arg3 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4 >
    inline void warning( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 )
  {
    warning( line, ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 ).str().c_str() );
  }

  template < typename T1 >
    inline void error( const char* str, T1 const& arg1 )
  {
    error( ( boost::format( str ) % arg1 ).str().c_str() );
  }

  template < typename T1, typename T2 >
    inline void error( const char* str, T1 const& arg1, T2 const& arg2 )
  {
    error( ( boost::format( str ) % arg1 % arg2 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3 >
    inline void error( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3 )
  {
    error( ( boost::format( str ) % arg1 % arg2 % arg3 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4 >
    inline void error( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 )
  {
    error( ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4, typename T5 >
    inline void error( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 , T5 const& arg5 )
  {
    error( ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 % arg5 ).str().c_str() );
  }
  
  template < typename T1 >
    inline void error( text_line const& line, const char* str, T1 const& arg1 )
  {
    error( line, ( boost::format( str ) % arg1 ).str().c_str() );
  }

  template < typename T1, typename T2 >
    inline void error( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2 )
  {
    error( line, ( boost::format( str ) % arg1 % arg2 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3 >
    inline void error( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3 )
  {
    error( line, ( boost::format( str ) % arg1 % arg2 % arg3 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4 >
    inline void error( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 )
  {
    error( line, ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 ).str().c_str() );
  }

  template < typename T1 >
    inline void fatal( const char* str, T1 const& arg1 )
  {
    fatal( ( boost::format( str ) % arg1 ).str().c_str() );
  }

  template < typename T1, typename T2 >
    inline void fatal( const char* str, T1 const& arg1, T2 const& arg2 )
  {
    fatal( ( boost::format( str ) % arg1 % arg2 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3 >
    inline void fatal( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3 )
  {
    fatal( ( boost::format( str ) % arg1 % arg2 % arg3 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4 >
    inline void fatal( const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 )
  {
    fatal( ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 ).str().c_str() );
  }

  template < typename T1 >
    inline void fatal( text_line const& line, const char* str, T1 const& arg1 )
  {
    fatal( line, ( boost::format( str ) % arg1 ).str().c_str() );
  }

  template < typename T1, typename T2 >
    inline void fatal( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2 )
  {
    fatal( line, ( boost::format( str ) % arg1 % arg2 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3 >
    inline void fatal( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3 )
  {
    fatal( line, ( boost::format( str ) % arg1 % arg2 % arg3 ).str().c_str() );
  }

  template < typename T1, typename T2, typename T3, typename T4 >
    inline void fatal( text_line const& line, const char* str, T1 const& arg1, T2 const& arg2, T3 const& arg3, T4 const& arg4 )
  {
    fatal( line, ( boost::format( str ) % arg1 % arg2 % arg3 % arg4 ).str().c_str() );
  }

  inline void exit()
  {
    throw normal_exit();
  }

#undef  _
#define _( str )  ::toppers::gettext( str ).c_str()

}

#endif  // ! TOPPERS_DIAGNOSTICS_HPP_

